@@ -2,7 +2,7 @@
 ### Fit a semiparametric bernoulli regression model.
 ###
 ### Copyright: Alejandro Jara Vallejos, 2006
-### Last modification: 05-07-2006.
+### Last modification: 21-08-2006.
 ###
 ### This program is free software; you can redistribute it and/or modify
 ### it under the terms of the GNU General Public License as published by
@@ -31,12 +31,13 @@
 ###
 
 "CSDPbinary"<-
-function(formula,prior,mcmc,state,status,misc=NULL,
+function(formula,baseline="logistic",prior,mcmc,state,status,misc=NULL,
 data=sys.frame(sys.parent()),na.action=na.fail) 
 UseMethod("CSDPbinary")
 
 "CSDPbinary.default"<-
 function(formula,
+         baseline="logistic",
          prior,
          mcmc,
          state,
@@ -119,7 +120,16 @@ function(formula,
          }
          
 
-         mcmcvec<-c(mcmc$nburn,mcmc$nskip,mcmc$ndisplay,mcmc$ntheta,model)
+         if(is.null(mcmc$ntheta))
+         {
+            ntheta<-1
+         }
+         else
+         {
+            ntheta<-mcmc$ntheta
+         }
+
+         mcmcvec<-c(mcmc$nburn,mcmc$nskip,mcmc$ndisplay,ntheta,model)
          nsave<-mcmc$nsave
           
          xmatrix<-x
@@ -186,7 +196,7 @@ function(formula,
          # output
          #########################################################################################
          
-         xlink=rev(seq(-6,6,length=34))
+         xlink=sort(c(seq(-6,6,length=34),0),decreasing=TRUE)
          nlink<-length(xlink)
          cpo<-rep(0,nrec)
          fsave     <- matrix(0, nrow=nsave, ncol=nlink)         
@@ -197,7 +207,8 @@ function(formula,
          # working space
          #########################################################################################
 
-	 ntotal<-nrec
+         #ntotal<-nrec
+         ntotal<-nrec+nlink
 	 maxint<-2*(ntotal+3)+1
 	 maxend<-2*(ntotal+3)
 	 maxm<-log(0.0001)/log((ntotal+1)/(ntotal+2))
@@ -244,6 +255,9 @@ function(formula,
          #########################################################################################
          # calling the fortran code
          #########################################################################################
+ 
+         if(baseline=="logistic")
+         {
 
    	    foo <- .Fortran("csdpbinaryl2",
 	 	nrec      =as.integer(nrec),
@@ -312,6 +326,7 @@ function(formula,
 		workv2    =as.double(workv2),
 		wvec      =as.double(wvec),
 		PACKAGE="DPpackage")	
+         }
 
          #########################################################################################
          # save state
@@ -324,11 +339,9 @@ function(formula,
          model.name<-"Bayesian semiparametric binary regression model"		
 
   	 colnames(thetasave)<-c(dimnames(x)[[2]],"theta","ncluster","alpha")
-  	 
-  	 coeff<-rep(0,(p+3))
-	 for(i in 1:(p+3)){
-	     coeff[i]<-mean(thetasave[,i])
-	 }  
+
+         coeff<-apply(thetasave, 2, mean)
+
 	 names(coeff)<-c(dimnames(x)[[2]],"theta","ncluster","alpha")
 	
          qnames<-NULL
@@ -339,7 +352,7 @@ function(formula,
          }
          qnames<-c(qnames,"Prediction")
          colnames(randsave)<-qnames
-	
+
 	 state <- list(beta=foo$beta,v=foo$v,y=foo$y,alpha=foo$alpha,theta=foo$theta)
 				  
 	 save.state <- list(thetasave=thetasave,fsave=fsave,randsave=randsave)
@@ -347,7 +360,7 @@ function(formula,
 	 z<-list(modelname=model.name,coefficients=coeff,acrate=foo$acrate,call=cl,
 	         prior=prior,mcmc=mcmcvec,state=state,save.state=save.state,nrec=foo$nrec,
 	         cpo=foo$cpo,p=p,nlink=nlink,xlink=xlink,x=x,
-	         ppar=prior$p,d=prior$d)
+	         ppar=prior$p,d=prior$d,baseline=baseline)
 	         
 	 cat("\n\n")
 
@@ -355,55 +368,60 @@ function(formula,
 	 return(z) 
 }
 
+
 ###                    
 ### Estimate the probability curve for a fitted semiparametric binary 
-### regression model.
+### regression model (Falta optimizar. Probablemente codigo fortran).
 ###
 ### Copyright: Alejandro Jara Vallejos, 2006
-### Last modification: 01-07-2006.
+### Last modification: 21-08-2006.
 
 predict.CSDPbinary<-function(object,xnew=NULL,hpd=TRUE, ...)
 {
+   stde<-function(x)
+   {
+   	n<-length(x)
+   	return(sd(x)/sqrt(n))
+   }
+
+   hpdf<-function(x)
+   {
+        alpha<-0.05
+        vec<-x
+        n<-length(x)         
+        alow<-rep(0,2)
+        aupp<-rep(0,2)
+        a<-.Fortran("hpd",n=as.integer(n),alpha=as.double(alpha),x=as.double(vec),
+                    alow=as.double(alow),aupp=as.double(aupp),PACKAGE="DPpackage")
+        return(c(a$alow[1],a$aupp[1]))
+   }
+   
+   pdf<-function(x)
+   {
+        alpha<-0.05
+        vec<-x
+        n<-length(x)         
+        alow<-rep(0,2)
+        aupp<-rep(0,2)
+        a<-.Fortran("hpd",n=as.integer(n),alpha=as.double(alpha),x=as.double(vec),
+                    alow=as.double(alow),aupp=as.double(aupp),PACKAGE="DPpackage")
+        return(c(a$alow[2],a$aupp[2]))
+   }
+
    if(is.null(xnew))
    {
       xnew<-object$x
    }
-   
-   pbaselinel<-function(x,theta,d,p)
-   {
-       if(x<=(theta-d))
-       {
-          area<-plogis(theta-d)
-          out<-((1-p)/2)*plogis(x)/area
-       }
 
-       if(x>(theta-d) && x<=0)
-       {
-          area<-0.5-plogis(theta-d)
-          out<-(1-p)/2+ (p/2)*plogis(x/area)
-       }
-
-       if(x>0 && x<=theta)
-       {
-          area<-plogis(theta)-0.5
-          out<-0.5+ (p/2)*plogis(x/area)
-       }
-
-       if(x>theta)
-       {
-          area<-1-plogis(theta)
-          out<-(1+p)/2+ ((1-p)/2)*plogis(x/area)
-       }
-       return(out)
-   }
-	
    if(is(object, "CSDPbinary"))
    {
    	 npred<-dim(xnew)[1]
    	 pnew<-dim(xnew)[2]
    	 nrec<-object$nrec
 
+         baseline<-object$baseline
    	 alpha<-object$save.state$thetasave[,(object$p+3)]
+   	 nsave<-length(alpha)
    	 theta<-object$save.state$thetasave[,(object$p+1)]
    	 ppar<-object$ppar
    	 d<-object$d
@@ -430,74 +448,73 @@ predict.CSDPbinary<-function(object,xnew=NULL,hpd=TRUE, ...)
          }    
 
          lp<-xnew%*%t(object$save.state$thetasave[,1:object$p]) 
+         out<-matrix(0,nrow=npred,ncol=nsave)
 
- 	 pm<-rep(0,npred)
-	 pmed<-rep(0,npred)
-	 psd<-rep(0,npred)
-	 pstd<-rep(0,npred)
-	 plinf<-rep(0,npred)
-	 plsup<-rep(0,npred)
-
-	 for(i in 1:npred)
+         if(baseline=="logistic")
          {
-	     prob<-rep(0,nsave)
-	     for(k in 1:nsave)
-	     {
-		lsup<-lp[i,k]
-		awork<-alpha[k]
-		vwork<-v[k,]
-		Cdelta<- sum(vwork<=lsup)
-		Cparam<- awork*pbaselinel(lsup,theta[k],d,ppar)
-		prob[k]<-(Cparam+Cdelta)/(awork+nrec)	                
-	      }
-	      pm[i]<-mean(prob)
-	      pmed[i]<-median(prob)
-	      psd[i]<-sqrt(var(prob))
-	      pstd[i]<-sqrt(var(prob))/sqrt(nsave)
-		 
-              if(hpd==TRUE)
-              {
-                    alow<-rep(0,2)
-                    aupp<-rep(0,2)
-                    sig<-0.05
-                    vec<-prob
-                    n<-length(vec)
-                    a<-.Fortran("hpd",n=as.integer(n),alpha=as.double(sig),x=as.double(vec),
-                                 alow=as.double(alow),aupp=as.double(aupp),PACKAGE="DPpackage")
-                    plinf[i]<-a$alow[1]            
-                    plsup[i]<-a$aupp[1]            
-              }
-              else
-              {
-		    plinf[i]<-quantile(surv[j,],0.025)		
-		    plsup[i]<-quantile(surv[j,],0.975)			                
-              }
-          }    
+   	    foo <- .Fortran("csdppredl",
+                nsave     =as.integer(nsave),
+                nrec      =as.integer(nrec),
+                npred     =as.integer(npred),
+                alpha     =as.double(alpha),
+                theta     =as.double(theta),
+                v         =as.double(v),
+                d         =as.double(d),
+                ppar      =as.double(ppar),
+                lp        =as.double(lp),
+		out       =as.double(out),
+		PACKAGE="DPpackage")	
+         }
 
-  	 
-   	  names(pm)<-covn
-   	  names(pmed)<-covn
-   	  names(psd)<-covn
-   	  names(pstd)<-covn
-   	  names(plinf)<-covn
-   	  names(plsup)<-covn
+         out<-t(matrix(foo$out,nrow=npred,ncol=nsave))
+
+         pm <-apply(out, 2, mean)    
+         pmed <-apply(out, 2, median)    
+         psd<-apply(out, 2, sd)
+         pstd<-apply(out, 2, stde)
+
+         if(hpd){             
+            limm<-apply(out, 2, hpdf)
+            plinf<-limm[1,]
+            plsup<-limm[2,]
+         }
+         else
+         {
+            plinf<-apply(out, 2, pdf)
+            coef.l<-limm[1,]
+            plsup<-limm[2,]
+         }
+
+   	 names(pm)<-covn
+   	 names(pmed)<-covn
+   	 names(psd)<-covn
+   	 names(pstd)<-covn
+   	 names(plinf)<-covn
+   	 names(plsup)<-covn
    	 
-   	  out<-NULL
-   	  out$pmean<-pm
-   	  out$pmedian<-pmed
-   	  out$psd<-psd
-   	  out$pstd<-pstd
-   	  out$plinf<-plinf
-   	  out$plsup<-plsup
-   	  out$vpred<-vpred
-   	  out$npred<-npred
+   	 out<-NULL
+   	 out$pmean<-pm
+   	 out$pmedian<-pmed
+   	 out$psd<-psd
+   	 out$pstd<-pstd
+   	 out$plinf<-plinf
+   	 out$plsup<-plsup
+   	 out$vpred<-vpred
+   	 out$npred<-npred
 
-   	  out$covn<-covn
+   	 out$covn<-covn
    }
    return(out)   
 }
 
 
+###
+### Tools for CSDPbinary: print, summary, plot
+###
+### Copyright: Alejandro Jara Vallejos, 2006
+### Last modification: 21-08-2006.
+
+
 "print.CSDPbinary"<-function (x, digits = max(3, getOption("digits") - 3), ...) 
 {
     cat("\n",x$modelname,"\n\nCall:\n", sep = "")
@@ -523,118 +540,138 @@ predict.CSDPbinary<-function(object,xnew=NULL,hpd=TRUE, ...)
 
 "summary.CSDPbinary"<-function(object, hpd=TRUE, ...) 
 {
-    dimen<-object$p
-    coef.p<-object$coefficients[1:dimen]
-    coef.sd<-rep(0,dimen)
-    coef.se<-rep(0,dimen)
-    coef.l<-rep(0,dimen)
-    coef.u<-rep(0,dimen)
-    coef.m<-rep(0,dimen)
-    names(coef.sd)<-names(object$coefficients[1:dimen])
-    names(coef.l)<-names(object$coefficients[1:dimen])
-    names(coef.u)<-names(object$coefficients[1:dimen])
-    
-    alpha<-0.05
-    
-    for(i in 1:dimen){
-        alow<-rep(0,2)
-        aupp<-rep(0,2)
-        coef.sd[i]<-sqrt(var(object$save.state$thetasave[,i]))
-        coef.m[i]<-median(object$save.state$thetasave[,i])
-        vec<-object$save.state$thetasave[,i]
-        n<-length(vec)
-        
-        if(hpd==TRUE)
-        {
-        
-                a<-.Fortran("hpd",n=as.integer(n),alpha=as.double(alpha),x=as.double(vec),
-                                  alow=as.double(alow),aupp=as.double(aupp),PACKAGE="DPpackage")
-                coef.l[i]<-a$alow[1]            
-                coef.u[i]<-a$aupp[1]            
-         }
-         else
-         {
-                coef.l[i]<-quantile(vec,0.025) 
-                coef.u[i]<-quantile(vec,0.975) 
-         }
+
+    stde<-function(x)
+    {
+    	n<-length(x)
+    	return(sd(x)/sqrt(n))
     }
 
-    coef.se<-coef.sd/sqrt(n)
+    hpdf<-function(x)
+    {
+         alpha<-0.05
+         vec<-x
+         n<-length(x)         
+         alow<-rep(0,2)
+         aupp<-rep(0,2)
+         a<-.Fortran("hpd",n=as.integer(n),alpha=as.double(alpha),x=as.double(vec),
+                     alow=as.double(alow),aupp=as.double(aupp),PACKAGE="DPpackage")
+         return(c(a$alow[1],a$aupp[1]))
+    }
+    
+    pdf<-function(x)
+    {
+         alpha<-0.05
+         vec<-x
+         n<-length(x)         
+         alow<-rep(0,2)
+         aupp<-rep(0,2)
+         a<-.Fortran("hpd",n=as.integer(n),alpha=as.double(alpha),x=as.double(vec),
+                     alow=as.double(alow),aupp=as.double(aupp),PACKAGE="DPpackage")
+         return(c(a$alow[2],a$aupp[2]))
+    }
+
+    thetasave<-object$save.state$thetasave
+
+
+### Fixed part of the model
+
+    dimen1<-object$p
+
+    if(dimen1==1)
+    {
+       mat<-matrix(thetasave[,1:dimen1],ncol=1) 
+    }
+    else
+    {
+       mat<-thetasave[,1:dimen1]
+    }
+
+    coef.p<-object$coefficients[1:dimen1]
+    coef.m <-apply(mat, 2, median)    
+    coef.sd<-apply(mat, 2, sd)
+    coef.se<-apply(mat, 2, stde)
+
+    if(hpd){             
+         limm<-apply(mat, 2, hpdf)
+         coef.l<-limm[1,]
+         coef.u<-limm[2,]
+    }
+    else
+    {
+         limm<-apply(mat, 2, pdf)
+         coef.l<-limm[1,]
+         coef.u<-limm[2,]
+    }
+
+    names(coef.m)<-names(object$coefficients[1:dimen1])
+    names(coef.sd)<-names(object$coefficients[1:dimen1])
+    names(coef.se)<-names(object$coefficients[1:dimen1])
+    names(coef.l)<-names(object$coefficients[1:dimen1])
+    names(coef.u)<-names(object$coefficients[1:dimen1])
 
     coef.table <- cbind(coef.p, coef.m, coef.sd, coef.se , coef.l , coef.u)
-    
-    if(hpd==TRUE)
+
+    if(hpd)
     {
        dimnames(coef.table) <- list(names(coef.p), c("Mean", "Median", "Std. Dev.", "Naive Std.Error",
                 "95%HPD-Low","95%HPD-Upp"))
-    }            
+    }
     else
     {
        dimnames(coef.table) <- list(names(coef.p), c("Mean", "Median", "Std. Dev.", "Naive Std.Error",
                 "95%CI-Low","95%CI-Upp"))
     }
     
-    
     ans <- c(object[c("call", "modelname")])
-    
+
     ans$coefficients<-coef.table
 
+
+### CPO
     ans$cpo<-object$cpo
     
 
+### Baseline Information
+
+
     if(is.null(object$prior$a0))
     {
-	dimen<-2	
+	dimen2<-2	
     }
     else
     {
-	dimen<-3	    
+	dimen2<-3	
     }
 
-    coef.p<-object$coefficients[(object$p+1):(object$p+dimen)]
-    coef.sd<-rep(0,dimen)
-    coef.se<-rep(0,dimen)
-    coef.l<-rep(0,dimen)
-    coef.u<-rep(0,dimen)
-    coef.m<-rep(0,dimen)
-    names(coef.sd)<-names(object$coefficients[(object$p+1):(object$p+dimen)])
-    names(coef.l)<-names(object$coefficients[(object$p+1):(object$p+dimen)])
-    names(coef.u)<-names(object$coefficients[(object$p+1):(object$p+dimen)])
-    for(i in 1:dimen){
-         alow<-rep(0,2)
-         aupp<-rep(0,2)
-         coef.sd[i]<-sqrt(var(object$save.state$thetasave[,(object$p+i)]))
-         coef.m[i]<-median(object$save.state$thetasave[,(object$p+i)])
-         vec<-object$save.state$thetasave[,(object$p+i)]
-         n<-length(vec)
-        
-         if(hpd==TRUE)
-         {
-             a<-.Fortran("hpd",n=as.integer(n),alpha=as.double(alpha),x=as.double(vec),
-                              alow=as.double(alow),aupp=as.double(aupp),PACKAGE="DPpackage")
-             coef.l[i]<-a$alow[1]            
-             coef.u[i]<-a$aupp[1]            
-         }
-         else
-         {
-                coef.l[i]<-quantile(vec,0.025) 
-                coef.u[i]<-quantile(vec,0.975) 
-         }
-    }
+    mat<-thetasave[,(dimen1+1):(dimen1+dimen2)]
+    coef.p<-object$coefficients[(dimen1+1):(dimen1+dimen2)]
+    coef.m <-apply(mat, 2, median)    
+    coef.sd<-apply(mat, 2, sd)
+    coef.se<-apply(mat, 2, stde)
 
-    coef.se<-coef.sd/sqrt(n)
+    if(hpd){             
+         limm<-apply(mat, 2, hpdf)
+         coef.l<-limm[1,]
+         coef.u<-limm[2,]
+    }
+    else
+    {
+         limm<-apply(mat, 2, pdf)
+         coef.l<-limm[1,]
+         coef.u<-limm[2,]
+    }
 
     coef.table <- cbind(coef.p, coef.m, coef.sd, coef.se , coef.l , coef.u)
-
-    if(hpd==TRUE)
+    if(hpd)
     {
-       dimnames(coef.table) <- list(names(coef.p), c("Mean", "Median", "Std. Dev.", "Naive Std.Error",
-                "95%HPD-Low","95%HPD-Upp"))
-    }            
+         dimnames(coef.table) <- list(names(coef.p), c("Mean", "Median", "Std. Dev.", "Naive Std.Error",
+             "95%HPD-Low","95%HPD-Upp"))
+    }
     else
     {
-       dimnames(coef.table) <- list(names(coef.p), c("Mean", "Median", "Std. Dev.", "Naive Std.Error",
-                "95%CI-Low","95%CI-Upp"))
+         dimnames(coef.table) <- list(names(coef.p), c("Mean", "Median", "Std. Dev.", "Naive Std.Error",
+             "95%CI-Low","95%CI-Upp"))
     }
 
     ans$prec<-coef.table
@@ -756,6 +793,37 @@ fancydensplot<-function(x, hpd=TRUE, npts=200, xlab="", ylab="", main="",col="#b
         axis(2., at = round(seq(0,max(densy),length=5), 2.), labels = T,pos =min(densx))
 }
 
+    stde<-function(x)
+    {
+    	n<-length(x)
+    	return(sd(x)/sqrt(n))
+    }
+
+    hpdf<-function(x)
+    {
+         alpha<-0.05
+         vec<-x
+         n<-length(x)         
+         alow<-rep(0,2)
+         aupp<-rep(0,2)
+         a<-.Fortran("hpd",n=as.integer(n),alpha=as.double(alpha),x=as.double(vec),
+                     alow=as.double(alow),aupp=as.double(aupp),PACKAGE="DPpackage")
+         return(c(a$alow[1],a$aupp[1]))
+    }
+    
+    pdf<-function(x)
+    {
+         alpha<-0.05
+         vec<-x
+         n<-length(x)         
+         alow<-rep(0,2)
+         aupp<-rep(0,2)
+         a<-.Fortran("hpd",n=as.integer(n),alpha=as.double(alpha),x=as.double(vec),
+                     alow=as.double(alow),aupp=as.double(aupp),PACKAGE="DPpackage")
+         return(c(a$alow[2],a$aupp[2]))
+    }
+
+
 
     if(is(x, "CSDPbinary")){
         if(is.null(param))
@@ -769,7 +837,7 @@ fancydensplot<-function(x, hpd=TRUE, npts=200, xlab="", ylab="", main="",col="#b
            for(i in 1:(n-1)){
                title1<-paste("Trace of",pnames[i],sep=" ")
                title2<-paste("Density of",pnames[i],sep=" ")       
-               plot(x$save.state$thetasave[,i],type='l',main=title1,xlab="MCMC scan",ylab=" ")
+               plot(ts(x$save.state$thetasave[,i]),type='l',main=title1,xlab="MCMC scan",ylab=" ")
                if(pnames[i]=="ncluster")
 	       {
 	          hist(x$save.state$thetasave[,i],main=title2,xlab="values", ylab="probability",probability=TRUE)
@@ -788,42 +856,31 @@ fancydensplot<-function(x, hpd=TRUE, npts=200, xlab="", ylab="", main="",col="#b
            {
                title1<-paste("Trace of",pnames[n],sep=" ")
                title2<-paste("Density of",pnames[n],sep=" ")       
-               plot(x$save.state$thetasave[,n],type='l',main=title1,xlab="MCMC scan",ylab=" ")
+               plot(ts(x$save.state$thetasave[,n]),type='l',main=title1,xlab="MCMC scan",ylab=" ")
                fancydensplot(x$save.state$thetasave[,n],hpd=hpd,main=title2,xlab="values", ylab="density",col=col)
            }
            
            title1<-c("Predictive Error Density")
            title2<-c("Link Function")
            fancydensplot(x$save.state$randsave[,(x$nrec+1)],hpd=hpd,main=title1,xlab="values", ylab="density",col=col)
-           
-           pml<-rep(0,x$nlink)
-           pll<-rep(0,x$nlink)
-           plu<-rep(0,x$nlink)
-           alpha<-0.05
-           
-           for(i in 1:x$nlink)
+
+           pml <-apply(x$save.state$fsave, 2, mean)    
+           if(hpd){             
+                limm<-apply(x$save.state$fsave, 2, hpdf)
+                pll<-limm[1,]
+                plu<-limm[2,]
+           }
+           else
            {
-               pml[i]<-mean(x$save.state$fsave[,i])
-               alow<-rep(0,2)
-	       aupp<-rep(0,2)
-	       vec<-x$save.state$fsave[,i]
-	       n<-length(vec)
-	       a<-.Fortran("hpd",n=as.integer(n),alpha=as.double(alpha),x=as.double(vec),
-		                 alow=as.double(alow),aupp=as.double(aupp),PACKAGE="DPpackage")
-	       if(hpd)
-	       {
-	           pll[i]<-a$alow[1]  
-	           plu[i]<-a$aupp[1]
-	       }
-	       else
-	       {
-	           pll[i]<-a$alow[2]  
-	           plu[i]<-a$aupp[2]
-	       }
-            }
-            plot(x$xlink,pml,xlab="x",ylab="probability",main=title2,lty=1,type='l',lwd=2,ylim=c(0,1))
-            lines(x$xlink,pll,lty=2,lwd=2)
-            lines(x$xlink,plu,lty=2,lwd=2)
+                limm<-apply(x$save.state$fsave, 2, pdf)
+                pll<-limm[1,]
+                plu<-limm[2,]
+           } 
+       
+           plot(x$xlink,pml,xlab="x",ylab="probability",main=title2,lty=1,type='l',lwd=2,ylim=c(0,1))
+           lines(x$xlink,pll,lty=2,lwd=2)
+           lines(x$xlink,plu,lty=2,lwd=2) 
+
         }
         else
         {
@@ -847,7 +904,7 @@ fancydensplot<-function(x, hpd=TRUE, npts=200, xlab="", ylab="", main="",col="#b
 	    {
                title1<-paste("Trace of",pnames[poss],sep=" ")
                title2<-paste("Density of",pnames[poss],sep=" ")       
-               plot(x$save.state$thetasave[,poss],type='l',main=title1,xlab="MCMC scan",ylab=" ")
+               plot(ts(x$save.state$thetasave[,poss]),type='l',main=title1,xlab="MCMC scan",ylab=" ")
                fancydensplot(x$save.state$thetasave[,poss],hpd=hpd,main=title2,xlab="values", ylab="density",col=col)
             }
             else
@@ -856,34 +913,22 @@ fancydensplot<-function(x, hpd=TRUE, npts=200, xlab="", ylab="", main="",col="#b
                title2<-c("Link Function")
                fancydensplot(x$save.state$randsave[,(x$nrec+1)],hpd=hpd,main=title1,xlab="values", ylab="density",col=col)
            
-               pml<-rep(0,x$nlink)
-               pll<-rep(0,x$nlink)
-               plu<-rep(0,x$nlink)
-               alpha<-0.05
-           
-               for(i in 1:x$nlink)
+               pml <-apply(x$save.state$fsave, 2, mean)    
+               if(hpd){             
+                    limm<-apply(x$save.state$fsave, 2, hpdf)
+                    pll<-limm[1,]
+                    plu<-limm[2,]
+               }
+               else
                {
-                   pml[i]<-mean(x$save.state$fsave[,i])
-                   alow<-rep(0,2)
-	           aupp<-rep(0,2)
-	           vec<-x$save.state$fsave[,i]
-	           n<-length(vec)
-	           a<-.Fortran("hpd",n=as.integer(n),alpha=as.double(alpha),x=as.double(vec),
-		                 alow=as.double(alow),aupp=as.double(aupp),PACKAGE="DPpackage")
-	           if(hpd)
-	           {
-	               pll[i]<-a$alow[1]  
-	               plu[i]<-a$aupp[1]
-	           }
-	           else
-	           {
-	              pll[i]<-a$alow[2]  
-	              plu[i]<-a$aupp[2]
-	           }
-                }
-                plot(x$xlink,pml,xlab="x",ylab="probability",main=title2,lty=1,type='l',lwd=2,ylim=c(0,1))
-                lines(x$xlink,pll,lty=2,lwd=2)
-                lines(x$xlink,plu,lty=2,lwd=2)
+                    limm<-apply(x$save.state$fsave, 2, pdf)
+                    pll<-limm[1,]
+                    plu<-limm[2,]
+               } 
+               plot(x$xlink,pml,xlab="x",ylab="probability",main=title2,lty=1,type='l',lwd=2,ylim=c(0,1))
+               lines(x$xlink,pll,lty=2,lwd=2)
+               lines(x$xlink,plu,lty=2,lwd=2)            
+
             }
         }
    }
