@@ -1,27 +1,70 @@
     
 c=======================================================================                      
-      subroutine dpsurvint(nrec,p,x,y,interind,
-     &                     mcmc,nsave,propv,
-     &                     a0b0,betapm,betapv,m0,s0,tau,
-     &                     acrate,thetasave,randsave,cpo,
-     &                     ncluster,alpha,beta,mu,sigma2,v,
-     &                     maxint,maxend,maxm,
-     &                     iflag,intcount,intcount2,    
-     &                     intind,intind2,seed,
-     &                     betac,eta,etan,endp,endp2,mass,
-     &                     prob,prob2,
-     &                     uvec,vvec,vnew,vnew2,wvec,
-     &                     workm1,workm2,
-     &                     workmh1,workv1,workv2)      
+      subroutine dpsurvint2(nrec,p,x,y,interind,
+     &                      mcmc,nsave,propv,
+     &                      a0b0,betapm,betapv,m0,s0,tau,
+     &                      acrate,thetasave,randsave,cpo,
+     &                      ncluster,alpha,beta,mu,sigma2,v,
+     &                      maxint,maxend,maxm,
+     &                      iflag,
+     &                      imaxs,imaxsc,imins,iminsc,
+     &                      intcount,intcount2,    
+     &                      intind,intind2,
+     &                      intposso,intpossn,
+     &                      seed,
+     &                      betac,eta,etan,endp,endp2,
+     &                      linfs,linfsc,lsups,lsupsc,
+     &                      mass,
+     &                      prob,
+     &                      uvec,vvec,vnew,vnew2,wvec,
+     &                      workm1,workm2,
+     &                      workmh1,workv1,workv2)      
 c=======================================================================                      
 c
-c     Version 1.0: 
-c     Last modification: 02-05-2006.
-c
-c     Subroutine `dpsurvint' to run a Markov chain in the  
+c     Subroutine `dpsurvint2' to run a Markov chain in the  
 c     semiparametric atf model for interval censored data. 
 c
-c     Author: A.J.V.
+c     Copyright: Alejandro Jara Vallejos, 2006
+c
+c     Version 2.0: 
+c
+c     Last modification: 02-07-2006.
+c
+c     Changes and Bug fixes: 
+c
+c     Version 1.0 to Version 2.0:
+c          - Uses qsort3 subroutine to sort errors and endpoints.
+c          - Keeps the observations that belong to each interval
+c            in memory to allow a faster assignment of the errors.
+c          - Performs a binary search to find the endpoints corresponding
+c            to each interval.
+c
+c     This program is free software; you can redistribute it and/or modify
+c     it under the terms of the GNU General Public License as published by
+c     the Free Software Foundation; either version 2 of the License, or (at
+c     your option) any later version.
+c
+c     This program is distributed in the hope that it will be useful, but
+c     WITHOUT ANY WARRANTY; without even the implied warranty of
+c     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+c     General Public License for more details.
+c
+c     You should have received a copy of the GNU General Public License
+c     along with this program; if not, write to the Free Software
+c     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+c
+c     The author's contact information:
+c
+c     Alejandro Jara Vallejos
+c     Biostatistical Centre
+c     Katholieke Universiteit Leuven
+c     U.Z. Sint-Rafal
+c     Kapucijnenvoer 35
+c     B-3000 Leuven
+c     Voice: +32 (0)16 336892 
+c     Fax  : +32 (0)16 337015 
+c     URL  : http://student.kuleuven.be/~s0166452/
+c     Email: Alejandro.JaraVallejos@med.kuleuven.be
 c
 c---- Data -------------------------------------------------------------
 c
@@ -108,6 +151,8 @@ c                       candidate for regression parameters, betac(p).
 c        cdflnorm    :  cdf of a lognormal distribution.
 c        counter     :  index.
 c        dispcount   :  index. 
+c        efind       :  integer function to find the intervals where the
+c                       errors should be sampled from.
 c        endp        :  real vector giving the end of the intervals,
 c                       endp(maxend).
 c        endp2       :  real vector giving the end of the intervals,
@@ -118,6 +163,24 @@ c        etan        :  real vector giving the linear predictor,
 c                       etan(nrec).
 c        evali       :  integer indicator used in updating the state.
 c        i           :  index.
+c        iflag       :  integer vector used to evaluate the prior
+c                       distribution for the regression coefficients, 
+c                       iflag(p).
+c        imaxs       :  integer vector used to store the highest
+c                       partition where each error can be sampled from,
+c                       imaxs(nrec). 
+c        imaxsc      :  integer vector used to store the highest
+c                       partition defined for the candidate coefficients
+c                       where each error can be sampled from,
+c                       imaxs(nrec). 
+c        imins       :  integer vector used to store the lowest
+c                       partition where each error can be sampled from,
+c                       imins(nrec). 
+c        iminsc      :  integer vector used to store the lowest
+c                       partition defined for the candidate coefficients
+c                       where each error can be sampled from,
+c                       imins(nrec). 
+
 c        indi        :  index.
 c        intcount    :  integer vector used to count the number of 
 c                       observations for each interval, 
@@ -129,22 +192,35 @@ c        intind      :  integer vector giving the interval where
 c                       each obbservation belong, intind(nrec+1).
 c        intind2     :  integer vector giving the interval where 
 c                       each obbservation belong, intind2(nrec+1).
-c        iflag       :  integer vector used to evaluate the prior
-c                       distribution for the regression coefficients, 
-c                       iflag(p).
+c        intposso    :  integer matrix giving the observations that
+c                       belong to each cluster, intposso(maxint,nrec+1).
+c        intpossn    :  integer matrix giving the observations that
+c                       belong to each cluster, intpossn(maxint,nrec+1).
 c        isave       :  index. 
 c        iscan       :  index. 
 c        j           :  index. 
-c        j1          :  index.         
-c        j2          :  index.         
 c        k           :  index.   
 c        keepbeta    :  real working variable.
 c        liminf      :  real working variable.
 c        limsup      :  real working variable.
+c        linfs       :  real vector used to store the lim inf of the
+c                       interval where the error must be sampled,
+c                       linfs(nrec).
+c        linfsc      :  real vector used to store the lim inf of the
+c                       interval defined for the candidate value of the 
+c                       coefficients where the error must be sampled, 
+c                       linfsc(nrec).
 c        logliko     :  real working variable.
 c        loglikn     :  real working variable.
 c        logprioro   :  real working variable.
 c        logpriorn   :  real working variable.
+c        lsups       :  real vector used to store the lim sup of the
+c                       interval where the error must be sampled,
+c                       lsups(nrec).
+c        lsupsc      :  real vector used to store the lim sup of the
+c                       interval defined for the candidate value of the 
+c                       coefficients where the error must be sampled, 
+c                       lsupsc(nrec).
 c        maxend      :  integer giving the maximum number of intervals.
 c        maxm        :  integer giving the maximum number of points in
 c                       the finite approximation of the DP.
@@ -160,8 +236,6 @@ c        nscan       :  index.
 c        ok          :  integer indicator.
 c        prob        :  real vector used to update the cluster 
 c                       structure, prob(maxint).
-c        prob2       :  real vector used to update the cluster 
-c                       structure, prob2(maxint).
 c        ratio       :  real working variable.
 c        rbeta       :  real beta random number generator.
 c        rgamma      :  real gamma random number generator.
@@ -171,7 +245,6 @@ c        rtlnorm     :  real truncated lnormal random number generator.
 c        seed1       :  seed for random number generation.
 c        seed2       :  seed for random number generation.
 c        seed3       :  seed for random number generation.
-c        sel         :  index.
 c        skipcount   :  index. 
 c        swork       :  real working variable.
 c        tmp1        :  real used to accumulate quantities. 
@@ -219,17 +292,29 @@ c+++++Current values of the parameters
 
 c+++++Working space
       integer maxint 
-      integer counter,dispcount,evali,i,indi,intcount(maxint),iflag(p)
+      integer counter,dispcount
+      integer efind
+      integer evali
+      integer i,iflag(p)
+      integer imax,imaxs(nrec),imaxsc(nrec)
+      integer imin,imins(nrec),iminsc(nrec)
+      integer indi,intcount(maxint)
       integer intcount2(maxint),intind(nrec+1),intind2(nrec+1)
-      integer isave,iscan,j,j1,j2,k
+      integer intposso(maxint,nrec+1),intpossn(maxint,nrec+1)
+      integer isave,iscan,j,k
       integer maxend,maxm
       integer mrand,npoints,ns,nso,nscan,ok
       integer seed(3),seed1,seed2,seed3
-      integer sel,skipcount
+      integer skipcount
+      
       real*8 area,betac(p),cdflnorm
-      real*8 eta(nrec),etan(nrec),endp(maxend),endp2(maxend)
-      real*8 keepbeta,liminf,limsup,logliko,loglikn,logprioro,logpriorn
-      real*8 mass(maxint),maxu,prob(maxint),prob2(maxint)
+      real*8 eta(nrec),etan(nrec)
+      real*8 endp(maxend),endp2(maxend)
+      real*8 keepbeta
+      real*8 liminf,limsup,linfs(nrec),linfsc(nrec)
+      real*8 logliko,loglikn,logprioro,logpriorn
+      real*8 lsups(nrec),lsupsc(nrec)
+      real*8 mass(maxint),maxu,prob(maxint)
       real*8 ratio,rbeta,rgamma,rnorm,rtlnorm,sigma,swork,tmp1,tmp2
       real*8 uvec(maxm),workm1(p,p),workm2(p,p),workmh1(p*(p+1)/2)
       real*8 workv1(p),workv2(p),wvec(maxm),vbar,vvec(maxm),vnew(nrec+1)
@@ -238,6 +323,9 @@ c+++++Working space
       logical ainf,binf
       real runif
 
+c+++++CPU time
+      real*8 sec00,sec0,sec1,sec
+
 c++++ initialize variables
 
       nburn=mcmc(1)
@@ -253,7 +341,7 @@ c++++ initialize variables
       
       call rdisc(1,nrec,evali)
       vpred=v(evali)
-      sel=1
+
 
 c++++ set random number generator
 
@@ -261,13 +349,86 @@ c++++ set random number generator
       seed2=seed(2)
       seed3=seed(3)
 
-      call setrand(seed1,seed2,seed3)
-      
+      call setall(seed1,seed2)
 
-c++++ evaluate log-prior for current value of parameters
+c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+c++++ Check consistency with the data before starting 
+c++++ the algorithm
+c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
-      call dmvn(p,beta,betapm,betapv,logprioro,workv1,workm1,
-     &          workm2,workv2,iflag)  
+c++++ check if the user has requested an interrupt
+      call rchkusr()
+ 
+      do i=1,nrec
+         vnew(i)=v(i)
+         if(interind(i).eq.1)then
+           linfs(i)=0.0
+           lsups(i)=y(i,2)*exp(eta(i))
+
+           if(v(i).gt.lsups(i))then
+              call intpr("i",-1,i,1)
+              call intpr("type",-1,interind(i),1)
+              call dblepr("u",-1,y(i,2)*exp(eta(i)),1)
+              call dblepr("eta",-1,eta(i),1)
+              call dblepr("v",-1,v(i),1)
+              call rexit("Errors not consistent with data in S0")
+           end if   
+         end if
+         
+         if(interind(i).eq.3)then
+           linfs(i)=y(i,1)*exp(eta(i))
+           lsups(i)=0.0
+           if(v(i).le.linfs(i))then
+              call intpr("i",-1,i,1)
+              call intpr("type",-1,interind(i),1)
+              call dblepr("l",-1,y(i,1)*exp(eta(i)),1)
+              call dblepr("eta",-1,eta(i),1)
+              call dblepr("v",-1,v(i),1)
+              call rexit("Errors not consistent with data in S0")
+           end if   
+         end if
+            
+         if(interind(i).eq.2)then
+           linfs(i)=y(i,1)*exp(eta(i))
+           lsups(i)=y(i,2)*exp(eta(i))
+
+           if(v(i).le.linfs(i))then
+              call intpr("i",-1,i,1)
+              call intpr("type",-1,interind(i),1)
+              call dblepr("l",-1,y(i,1)*exp(eta(i)),1)
+              call dblepr("u",-1,y(i,2)*exp(eta(i)),1)
+              call dblepr("eta",-1,eta(i),1)
+              call dblepr("v",-1,v(i),1)
+              call rexit("Errors not consistent with data in S0")
+           end if   
+           if(v(i).gt.lsups(i))then
+              call intpr("i",-1,i,1)
+              call intpr("type",-1,interind(i),1)
+              call dblepr("l",-1,y(i,1)*exp(eta(i)),1)
+              call dblepr("u",-1,y(i,2)*exp(eta(i)),1)
+              call dblepr("eta",-1,eta(i),1)
+              call dblepr("v",-1,v(i),1)
+              call rexit("Errors not consistent with data in S0")
+           end if   
+         end if   
+      end do
+c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+c++++ Sort errors and compute the number of clusters 
+c++++ before starting the algorithm
+c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+
+      call sortvec(nrec+1,vnew,1,nrec)
+
+      ncluster=1
+      vnew2(1)=vnew(1)
+     
+      do i=2,nrec
+         call rchkusr()
+         if(vnew(i).ne.vnew2(ncluster))then
+            ncluster=ncluster+1
+            vnew2(ncluster)=vnew(i)
+         end if
+      end do
 
 
 c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
@@ -279,6 +440,14 @@ c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
       dispcount=0
       nscan=nburn+(nskip+1)*(nsave)
 
+      call cpu_time(sec0)
+      sec00=0.d0
+
+c++++ evaluate log-prior for current value of parameters
+
+      call dmvn(p,beta,betapm,betapv,logprioro,workv1,workm1,
+     &          workm2,workv2,iflag)  
+
       do iscan=1,nscan
 
 c+++++++ check if the user has requested an interrupt
@@ -298,7 +467,7 @@ c+++++++ evaluate log-prior for candidate value of parameters
      &             workm2,workv2,iflag)  
 
 c+++++++ define the end points base on data
-         
+
          counter=0
          do i=1,nrec
             tmp1=0.d0
@@ -309,6 +478,10 @@ c+++++++ define the end points base on data
             etan(i)=tmp1
             
             if(interind(i).eq.1)then
+               linfs(i)=0.0
+               lsups(i)=y(i,2)*exp(eta(i))
+               linfsc(i)=0.0
+               lsupsc(i)=y(i,2)*exp(etan(i))
                counter=counter+1
                endp(counter)=y(i,2)*exp(eta(i))   
                counter=counter+1
@@ -316,6 +489,10 @@ c+++++++ define the end points base on data
             end if
             
             if(interind(i).eq.3)then
+               linfs(i)=y(i,1)*exp(eta(i))
+               lsups(i)=0.0
+               linfsc(i)=y(i,1)*exp(etan(i))
+               lsupsc(i)=0.0
                counter=counter+1
                endp(counter)=y(i,1)*exp(eta(i))   
                counter=counter+1
@@ -323,6 +500,10 @@ c+++++++ define the end points base on data
             end if
             
             if(interind(i).eq.2)then
+               linfs(i)=y(i,1)*exp(eta(i))
+               lsups(i)=y(i,2)*exp(eta(i))
+               linfsc(i)=y(i,1)*exp(etan(i))
+               lsupsc(i)=y(i,2)*exp(etan(i))
                counter=counter+1
                endp(counter)=y(i,1)*exp(eta(i))   
                counter=counter+1
@@ -336,15 +517,7 @@ c+++++++ define the end points base on data
          
 c+++++++ sort end points
 
-         do i=1,counter-1
-            do j=i+1,counter
-               if(endp(j).lt.endp(i))then
-                  tmp1=endp(i)
-                  endp(i)=endp(j)
-                  endp(j)=tmp1
-               end if
-            end do
-         end do   
+         call sortvec(maxend,endp,1,counter)
          
          npoints=1
          endp2(1)=endp(1)
@@ -355,11 +528,8 @@ c+++++++ sort end points
             end if
          end do
 
-c         call intpr("npoints",-1,npoints,1)
-
 c+++++++ updating G base on the partition defined by beta and betac
 
-
          do i=1,maxint
             mass(i)=0.d0
             intcount2(i)=0
@@ -373,16 +543,10 @@ c+++++++ updating G base on the partition defined by beta and betac
             mass(indi)=mass(indi)+1.d0
             intind2(i)=indi
             intcount2(indi)=intcount2(indi)+1
+            
+            intposso(indi,intcount2(indi))=i
          end do
 
-c+++++++ pred
-         indi=npoints+1
-         do j=npoints,1,-1
-            if(vpred.le.endp2(j))indi=indi-1
-         end do
-         intcount2(indi)=intcount2(indi)+1
-c+++++++ end pred
-
          do i=1,npoints+1
             area=0.d0
             if(i.eq.1)then
@@ -400,10 +564,6 @@ c+++++++ end pred
          call dirichlet(mass,maxint,npoints+1,prob)
          
 
-c         call dblepr("indi",-1,endp2,npoints)
-         
-c         return
-
 c+++++++ evaluate log-likelihood for current and candidate value 
 c+++++++ of parameters
 
@@ -414,68 +574,67 @@ c+++++++ of parameters
             tmp2=0.d0
 
             if(interind(i).eq.1)then
-               j=1 
-               do while(y(i,2)*exp(eta(i)).ge.endp2(j).and.j.le.npoints)   
+               imin=1
+               imax=efind(lsups(i),maxend,endp2,npoints)
+               tmp1=0.d0
+               do j=imin,imax
                   tmp1=tmp1+prob(j)
-                  j=j+1
                end do
+               imins(i)=imin
+               imaxs(i)=imax
 
-               j=1 
-               do while(y(i,2)*exp(etan(i)).ge.endp2(j)
-     &                  .and.j.le.npoints)   
+               imin=1
+               imax=efind(lsupsc(i),maxend,endp2,npoints)
+               tmp2=0.d0
+               do j=imin,imax
                   tmp2=tmp2+prob(j)
-                  j=j+1
                end do
-            end if            
-
+               iminsc(i)=imin
+               imaxsc(i)=imax
+            end if   
+            
             if(interind(i).eq.3)then
-               j=npoints 
-               do while(y(i,1)*exp(eta(i)).le.endp2(j).and.j.ge.1)   
-                  tmp1=tmp1+prob(j+1)
-                  j=j-1
+               imin=efind(linfs(i),maxend,endp2,npoints)
+               imin=imin+1
+               imax=npoints+1 
+               tmp1=0.d0
+               do j=imin,imax
+                  tmp1=tmp1+prob(j)
                end do
+               imins(i)=imin
+               imaxs(i)=imax
 
-               j=npoints 
-               do while(y(i,1)*exp(etan(i)).le.endp2(j).and.j.ge.1)   
-                  tmp2=tmp2+prob(j+1)
-                  j=j-1
+               imin=efind(linfsc(i),maxend,endp2,npoints)
+               imin=imin+1
+               imax=npoints+1 
+               tmp2=0.d0
+               do j=imin,imax
+                  tmp2=tmp2+prob(j)
                end do
-            end if            
+               iminsc(i)=imin
+               imaxsc(i)=imax
+            end if   
 
             if(interind(i).eq.2)then
-               j1=npoints 
-               do while(y(i,1)*exp(eta(i)).le.endp2(j1).and.j1.ge.1)   
-                  j1=j1-1
-               end do
-               j1=j1+1
-               
-               j2=1 
-               do while(y(i,2)*exp(eta(i)).ge.endp2(j2)
-     &                 .and.j2.le.npoints)   
-                  j2=j2+1
-               end do
-               j2=j2-1
-               
-               do j=j1,j2
+               imin=efind(linfs(i),maxend,endp2,npoints)
+               imin=imin+1
+               imax=efind(lsups(i),maxend,endp2,npoints)
+               tmp1=0.d0
+               do j=imin,imax
                   tmp1=tmp1+prob(j)
                end do
+               imins(i)=imin
+               imaxs(i)=imax
                
-               j1=npoints 
-               do while(y(i,1)*exp(etan(i)).le.endp2(j1).and.j1.ge.1)   
-                  j1=j1-1
-               end do
-               j1=j1+1
-               
-               j2=1 
-               do while(y(i,2)*exp(etan(i)).ge.endp2(j2)
-     &                  .and.j2.le.npoints)   
-                  j2=j2+1
-               end do
-               j2=j2-1
-               
-               do j=j1,j2
+               imin=efind(linfsc(i),maxend,endp2,npoints)
+               imin=imin+1
+               imax=efind(lsupsc(i),maxend,endp2,npoints)
+               tmp2=0.d0
+               do j=imin,imax
                   tmp2=tmp2+prob(j)
                end do
+               iminsc(i)=imin
+               imaxsc(i)=imax
             end if            
 
             logliko=logliko+log(tmp1)
@@ -495,6 +654,10 @@ c+++++++ aceptation step
             logprioro=logpriorn
             do i=1,nrec
                eta(i)=etan(i)
+               linfs(i)=linfsc(i)
+               lsups(i)=lsupsc(i)
+               imins(i)=iminsc(i)
+               imaxs(i)=imaxsc(i)
             end do
             acrate=acrate+1.d0
            else 
@@ -514,79 +677,63 @@ c+++++++ check if the user has requested an interrupt
 
          do i=1,maxint
             intcount(i)=0
-            prob2(i)=0.d0
          end do
 
          do i=1,nrec
-         
-            if(interind(i).eq.1)then
-               do j=1,npoints+1
-                  prob2(j)=0.d0
-               end do
-               
-               j=1 
-               do while(y(i,2)*exp(eta(i)).ge.endp2(j).and.j.le.npoints)   
-                  prob2(j)=prob(j)
-                  j=j+1
-               end do
-               
-               call simdisc(prob2,maxint,npoints+1,evali)
-            end if                              
+            imin=imins(i) 
+            imax=imaxs(i)
+            tmp1=0.d0
+            do j=imin,imax
+               tmp1=tmp1+prob(j)
+            end do
 
+            if(tmp1.eq.0.d0)then
+               call intpr("Imin",-1,imin,1)
+               call intpr("Imax",-1,imax,1)
+               call rexit("Zero Probability")
+              else
+               call simdiscint(prob,maxint,imin,imax,evali)               
+            end if   
 
-            if(interind(i).eq.3)then
-               do j=1,npoints+1
-                  prob2(j)=0.d0
-               end do
+            if(evali.lt.imin)then
+               call intpr("Evali",-1,evali,1)
+               call intpr("Imin",-1,imin,1)
+               call intpr("Imax",-1,imax,1)
+               call rexit("Error in the intervals")
+            end if  
+            if(evali.gt.imax)then
+               call intpr("Evali",-1,evali,1)
+               call intpr("Imin",-1,imin,1)
+               call intpr("Imax",-1,imax,1)
+               call rexit("Error in the intervals")
+            end if  
+            
+            if(evali.lt.1.and.evali.gt.(npoints+1))then
+               call intpr("Evali",-1,evali,1)
+               call intpr("Imin",-1,imin,1)
+               call intpr("Imax",-1,imax,1)
+               call rexit("Error in the intervals")
+            end if  
 
-               j=npoints 
-               do while(y(i,1)*exp(eta(i)).le.endp2(j).and.j.ge.1)   
-                  prob2(j+1)=prob(j+1)
-                  j=j-1
-               end do
-
-               call simdisc(prob2,maxint,npoints+1,evali)
-            end if            
-
-
-            if(interind(i).eq.2)then
-               do j=1,npoints+1
-                  prob2(j)=0.d0
-               end do
-               
-               j1=npoints 
-               do while(y(i,1)*exp(eta(i)).le.endp2(j1).and.j1.ge.1)   
-                  j1=j1-1
-               end do
-               j1=j1+1
-               
-               j2=1 
-               do while(y(i,2)*exp(eta(i)).ge.endp2(j2)
-     &                 .and.j2.le.npoints)   
-                  j2=j2+1
-               end do
-               j2=j2-1
-               
-               do j=j1,j2
-                  prob2(j)=prob(j)
-               end do
-               
-               call simdisc(prob2,maxint,npoints+1,evali)
-            end if            
-
-            if(evali.lt.1.and.evali.gt.npoints+1)return
             intcount(evali)=intcount(evali)+1
             intind(i)=evali
+
+            intpossn(evali,intcount(evali))=i
+            
+            imins(i)=imin
+            imaxs(i)=imax
+            
          end do
 
-c+++++++ pred         
-         do j=1,npoints+1
-	    prob2(j)=prob(j)
-         end do
-         call simdisc(prob2,maxint,npoints+1,evali)
+
+c+++++++ predictions         
+
+         call simdiscint(prob,maxint,1,npoints+1,evali)                        
          intcount(evali)=intcount(evali)+1
          intind(nrec+1)=evali
-c+++++++ end pred         
+         intpossn(evali,intcount(evali))=nrec+1
+
+c+++++++ end predictions         
 
          do i=1,npoints+1
             ns=intcount(i)
@@ -600,7 +747,7 @@ c+++++++ end pred
                liminf=0.d0
                limsup=endp2(1)
              else if(i.gt.npoints)then
-               area=alpha*(1.d0-cdflnorm(endp2(npoints),mu,sigma,1,0))
+               area=alpha*cdflnorm(endp2(npoints),mu,sigma,0,0)
                ainf=.false.
                binf=.true.
                liminf=endp2(npoints)
@@ -630,7 +777,7 @@ c+++++++ end pred
                do while(maxu.ge.tmp2.and.mrand.lt.maxm)
                   mrand=mrand+1
                   if(mrand.gt.maxm)then
-                    call intpr('Increase maxn in R function',-1,0,1)   
+                    call rexit("Increase maxn in R function")
                     return
                   end if  
                   keepbeta=rbeta(1.d0,mass(i))
@@ -638,44 +785,28 @@ c+++++++ end pred
                   tmp1=tmp1*(1.d0-keepbeta)
                   tmp2=tmp2+wvec(mrand)
                end do
-               
+
                do j=1,mrand
                   if(dble(runif()).le.(1.d0-(area/mass(i))))then
-                     if(nso.eq.0)return
                      call rdisc(1,nso,evali)
-                     counter=0
-                     k=1
-                     do while(k.le.(nrec+1).and.counter.lt.evali)
-                        if(intind2(k).eq.i)then
-                           counter=counter+1 
-                           sel=k
-                        end if
-                        k=k+1
-                     end do
-                     vvec(j)=v(sel)  
+                     if(evali.lt.1.or.evali.gt.nso)return
+                     vvec(j)=v(intposso(i,evali))
                    else
-                     vvec(j)=rtlnorm(mu,sigma,ainf,binf,liminf,limsup)
+                     tmp1=rtlnorm(mu,sigma,liminf,limsup,ainf,binf)
+                     vvec(j)=tmp1
                   end if
                end do
-               
-c               call dblepr("vvec",-1,vvec,mrand)
 
-               j=1
-               counter=0
-               do while(j.le.(nrec+1).and.counter.lt.ns)
-                  if(intind(j).eq.i)then
-	            counter=counter+1 
-	            tmp1=0.d0
-	            k=1
-	            do while(uvec(counter).gt.tmp1.and.k.lt.mrand)
-	               tmp1=tmp1+wvec(k)
-	               k=k+1  
-	            end do
-	            vnew(j)=vvec(k)
-                  end if
-                  j=j+1
+               do j=1,ns
+                  tmp1=0.d0
+	          k=1
+	          do while(uvec(j).gt.tmp1.and.k.lt.mrand)
+	             tmp1=tmp1+wvec(k)
+	             k=k+1  
+	          end do
+	          vnew(intpossn(i,j))=vvec(k)
                end do
-               
+
             end if 
          end do
 
@@ -684,6 +815,86 @@ c               call dblepr("vvec",-1,vvec,mrand)
          end do
          vpred=vnew(nrec+1)
 
+
+c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+c+++++++ Check consistency with the data    
+c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+
+c+++++++ check if the user has requested an interrupt
+         call rchkusr()
+ 
+         do i=1,nrec
+            if(interind(i).eq.1)then
+              if(v(i).gt.lsups(i))then
+                 call intpr("i",-1,i,1)
+                 call intpr("type",-1,interind(i),1)    
+                 call intpr("interval",-1,intind(i),1)                 
+                 call dblepr("u",-1,y(i,2)*exp(eta(i)),1)
+                 call dblepr("eta",-1,eta(i),1)
+                 call dblepr("v",-1,v(i),1)
+                 call rexit("Errors not consistent with data in S1")
+              end if   
+            end if
+         
+            if(interind(i).eq.3)then
+              if(v(i).le.linfs(i))then
+                 call intpr("i",-1,i,1)
+                 call intpr("type",-1,interind(i),1)                 
+                 call intpr("interval",-1,intind(i),1)                 
+                 call dblepr("l",-1,y(i,1)*exp(eta(i)),1)
+                 call intpr("int",-1,intind(i),1)
+                 call dblepr("eta",-1,eta(i),1)
+                 call dblepr("v",-1,v(i),1)
+                 call rexit("Errors not consistent with data in S1")
+              end if   
+            end if
+            
+            if(interind(i).eq.2)then
+              if(v(i).le.linfs(i))then
+                 call intpr("i",-1,i,1)
+                 call intpr("type",-1,interind(i),1)     
+                 call intpr("interval",-1,intind(i),1) 
+                 call intpr("imin",-1,imins(i),1) 
+                 call intpr("imax",-1,imaxs(i),1) 
+                 call dblepr("l",-1,y(i,1)*exp(eta(i)),1)
+                 call dblepr("u",-1,y(i,2)*exp(eta(i)),1)
+                 call intpr("int",-1,intind(i),1)
+                 call dblepr("eta",-1,eta(i),1)
+                 call dblepr("v",-1,v(i),1)
+                 call rexit("Errors not consistent with data in S1")
+              end if   
+              if(v(i).gt.lsups(i))then
+                 call intpr("i",-1,i,1)
+                 call intpr("type",-1,interind(i),1)                 
+                 call intpr("interval",-1,intind(i),1)                 
+                 call intpr("imin",-1,imins(i),1) 
+                 call intpr("imax",-1,imaxs(i),1) 
+                 call dblepr("l",-1,y(i,1)*exp(eta(i)),1)
+                 call dblepr("u",-1,y(i,2)*exp(eta(i)),1)
+                 call dblepr("eta",-1,eta(i),1)
+                 call dblepr("v",-1,v(i),1)
+                 call rexit("Errors not consistent with data in S1")
+              end if   
+            end if   
+         end do
+      
+c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+c+++++++ Sort errors and compute the number of clusters 
+c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+
+         call sortvec(nrec+1,vnew,1,nrec)
+
+         ncluster=1
+         vnew2(1)=vnew(1)
+     
+         do i=2,nrec
+            call rchkusr()
+            if(vnew(i).ne.vnew2(ncluster))then
+               ncluster=ncluster+1
+               vnew2(ncluster)=vnew(i)
+            end if
+         end do
+
 100      continue
 
 
@@ -693,43 +904,17 @@ c+++++++ distribution                                       +++
 c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
 
-c+++++++ sort errors
-
-c+++++++ check if the user has requested an interrupt
-         call rchkusr()
- 
-         do i=1,nrec
-            vnew(i)=v(i) 
-         end do
-
-         do i=1,nrec-1
-            do j=i+1,nrec
-               if(vnew(j).lt.vnew(i))then
-                  tmp1=vnew(i)
-                  vnew(i)=vnew(j)
-                  vnew(j)=tmp1
-               end if
-            end do
-         end do   
-         
 c+++++++ clusters and updating step
 
 c+++++++ check if the user has requested an interrupt
          call rchkusr()
          
          swork=0.d0
-         ncluster=1
-         vnew2(1)=vnew(1)
          
-         do i=2,nrec
-            if(vnew(i).ne.vnew2(ncluster))then
-               ncluster=ncluster+1
-               vnew2(ncluster)=vnew(i)
-               swork=swork+log(vnew(i))
-            end if
+         do i=1,ncluster
+            swork=swork+log(vnew2(i))
          end do
 
-         
          tmp1=(1.d0/s0)+(dble(ncluster)/sigma)
          
          vbar=(m0/s0)+(swork/sigma)
@@ -778,49 +963,18 @@ c+++++++++++++ cluster information
                thetasave(isave,p+4)=alpha
 
 c+++++++++++++ cpo, errors and predictive information
-               do j=1,nrec
+               do i=1,nrec
 
-                  randsave(isave,j)=v(j)
-                  
+                  randsave(isave,i)=v(i)
+
+                  imin=imins(i)
+                  imax=imaxs(i)
                   tmp1=0.d0
-                  if(interind(j).eq.1)then
-                     k=1 
-                     do while(y(j,2)*exp(eta(j)).ge.endp2(k)
-     &                        .and.k.le.npoints)   
-                        tmp1=tmp1+prob(k)
-                        k=k+1
-                     end do
-                  end if      
+                  do j=imin,imax
+                     tmp1=tmp1+prob(j)
+                  end do
 
-                  if(interind(j).eq.3)then
-                     k=npoints 
-                     do while(y(j,1)*exp(eta(j)).le.endp2(k).and.k.ge.1)   
-                        tmp1=tmp1+prob(k+1)
-                        k=k-1
-                     end do
-                  end if            
-
-                  if(interind(j).eq.2)then
-                     j1=npoints 
-                     do while(y(j,1)*exp(eta(j)).le.endp2(j1)
-     &                        .and.j1.ge.1)   
-                        j1=j1-1
-                     end do
-                     j1=j1+1
-               
-                     j2=1 
-                     do while(y(j,2)*exp(eta(j)).ge.endp2(j2)
-     &                        .and.j2.le.npoints)   
-                        j2=j2+1
-                     end do
-                     j2=j2-1
-               
-                     do k=j1,j2
-                        tmp1=tmp1+prob(k)
-                     end do
-                  end if     
-
-                  cpo(j)=cpo(j)+1.0d0/tmp1 
+                  cpo(i)=cpo(i)+1.0d0/tmp1 
                end do
                
                randsave(isave,nrec+1)=vpred
@@ -828,8 +982,11 @@ c+++++++++++++ cpo, errors and predictive information
 c+++++++++++++ print
                skipcount = 0
                if(dispcount.ge.ndisplay)then
-c                  call intpr("isave",5,isave,1)
-                  tmp1=sprint(isave,nsave)
+                  call cpu_time(sec1)
+                  sec00=sec00+(sec1-sec0)
+                  sec=sec00
+                  sec0=sec1
+                  tmp1=sprint(isave,nsave,sec)
                   dispcount=0
                end if   
 
@@ -851,4 +1008,3 @@ c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
       return
       end
 
-
