@@ -1,18 +1,17 @@
     
 c=======================================================================                      
-      subroutine ptlmp(maxm,nrec,p,x,y,
+      subroutine ptlmp(maxm,ngrid,nrec,p,x,y,
      &                 a0b0,betapm,betapv,tau,  
-     &                 mcmc,nsave,propv,
+     &                 mcmc,nsave,propv,tune2,tune3,
      &                 seed,
-     &                 acrate,randsave,thetasave,cpo,
+     &                 acrate,randsave,thetasave,cpo,f,
      &                 alpha,beta,mu,sigma2,v,
      &                 betac,iflag,vc,workm1,workm2,
-     &                 workmh1,workv1,workv2)
-
+     &                 workmh1,workv1,workv2,grid)
 c=======================================================================                      
 c
 c     Version 1.0: 
-c     Last modification: 27-06-2006.
+c     Last modification: 08-10-2006.
 c
 c     Subroutine `ptlmp' to run a Markov chain in the  
 c     semiparametric linear regression model using a partially 
@@ -49,6 +48,10 @@ c     Email: Alejandro.JaraVallejos@med.kuleuven.be
 c
 c---- Data -------------------------------------------------------------
 c
+c        maxm        :  integer giving the number of binary partitions
+c                       in the Finite Polya tree prior.
+c        ngrid       :  integer giving the size of the grid where
+c                       the density estimate is evaluated.
 c        nrec        :  integer giving the number of observations.
 c        p           :  integer giving the number of fixed coefficients.
 c        x           :  real matrix giving the design matrix for the 
@@ -84,6 +87,10 @@ c        nskip       :  integer giving the thinning interval.
 c        nsave       :  integer giving the number of scans to be saved.
 c        propv       :  real matrix giving the variance of the normal
 c                       proposal for the mh algorithm, propv(p,p).
+c        tune2       :  Metropolis tunnig parameter for the scale
+c                       parameter.
+c        tune3       :  Metropolis tunnig parameter for the precision
+c                       parameter.
 c        
 c-----------------------------------------------------------------------
 c
@@ -98,12 +105,14 @@ c
 c---- Output ----------------------------------------------------------- 
 c
 c        acrate      :  real giving the MH acceptance rate. 
+c        cpo         :  real giving the cpo, cpo(nrec).
+c        f           :  real vector giving the density estimate at the
+c                       grid, f(ngrid).
 c        randsave    :  real matrix containing the mcmc samples for
 c                       the errors and prediction,
 c                       randsave(nsave,nrec+1).
 c        thetasave   :  real vector containing the mcmc sample for the
 c                       regression parameters, betsave(nsave,p+2). 
-c        cpo         :  real giving the cpo, cpo(nrec).
 c
 c-----------------------------------------------------------------------
 c
@@ -136,6 +145,8 @@ c        denom       :  real working variable.
 c        dispcount   :  index. 
 c        dlnrm       :  density of a lognormal normal distribution.
 c        dnrm        :  density of a normal distribution.
+c        grid        :  real vector giving the grid where the density
+c                       estimate is evaluated, grid(ngrid).
 c        i           :  index.
 c        iflag       :  integer vector used to evaluate the prior
 c                       distribution for the regression coefficients, 
@@ -197,7 +208,7 @@ c+++++Partially specified Polya Trees parameter
       integer maxm
      
 c+++++Observed variables
-      integer nrec,p  
+      integer ngrid,nrec,p  
       real*8 x(nrec,p),y(nrec)
 
 c+++++Prior information
@@ -206,14 +217,14 @@ c+++++Prior information
       
 c+++++MCMC parameters
       integer mcmc(3),nburn,nskip,nsave,ndisplay
-      real*8 propv(p,p)
+      real*8 propv(p,p),tune2,tune3
 
 c+++++Random numbers
       integer seed(3),seed1,seed2,seed3
 
 c+++++Stored output
       real*8 acrate(3),randsave(nsave,nrec+1),thetasave(nsave,p+2)
-      real*8 cpo(nrec)
+      real*8 cpo(nrec),f(ngrid)
       
 c+++++Current values of the parameters
       real*8 alpha,beta(p),mu,sigma2,v(nrec)
@@ -240,6 +251,7 @@ c+++++Working space
       real*8 denom
       real*8 dlnrm
       real*8 dnrm
+      real*8 grid(ngrid)
       real*8 invcdfnorm
       real*8 linf
       real*8 logcgkn,logcgko,loglikec,loglikeo,logpriorc,logprioro
@@ -301,6 +313,111 @@ c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
       call cpu_time(sec0)
       sec00=0.d0
 
+c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+c++++ First computation of loglikelihood (to reduce CPU time)
+c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+
+      loglikeo=0.d0
+
+      do i=1,nrec
+         call rchkusr()   
+         
+         tmp1=0.d0
+         do j=1,p
+            tmp1=tmp1+x(i,j)*beta(j)
+         end do
+         v(i)=y(i)-tmp1
+      
+
+c+++++++ first observation
+         if(i.eq.1)then
+
+            loglikeo=dnrm(v(1) ,mu,sd ,1)
+
+c+++++++ following observations
+           else
+
+              quan=0.d0
+              countero=0
+              
+              if(v(i).le.quan) then
+                 do l=1,i-1
+                    if(v(l).le.quan)countero=countero+1
+                 end do
+               else
+                 do l=1,i-1
+                    if(v(l).gt.quan)countero=countero+1
+                 end do
+              end if  
+
+              if(countero.eq.0)go to 100
+              
+              ok=1
+              j=2
+              do while(ok.eq.1.and.j.le.maxm)
+                 nint=2**j
+                 je2=j**2
+                 prob=1.d0/dble(nint)
+                 k=1
+                 
+                 quan=invcdfnorm(prob,mu,sd,1,0)
+                 
+                 do while(v(i).gt.quan.and.k.le.(nint-1))
+                    k=k+1
+                    if(k.lt.nint)then
+                      quan=invcdfnorm(dble(k)*prob,mu,sd,1,0)
+                    end if  
+                 end do
+                 
+                 countern=0
+                 
+                 if(k.eq.1)then
+                    do l=1,i-1
+                       if(v(l).le.quan)countern=countern+1
+                    end do
+                  else if(k.eq.nint)then
+                    quan=invcdfnorm(dble(k-1)*prob,mu,sd,1,0) 
+                    do l=1,i-1
+                       if(v(l).gt.quan)countern=countern+1
+                    end do
+                  else
+                    tmp1=invcdfnorm(dble(k-1)*prob,mu,sd,1,0)
+                    tmp2=invcdfnorm(dble(k  )*prob,mu,sd,1,0)
+
+                    if(tmp1.ge.tmp2)then
+                      call rexit("Error in the limits")
+                    end if  
+                  
+                    do l=1,i-1
+                       if(v(l).gt.tmp1.and.v(l).le.tmp2)then
+                          countern=countern+1
+                       end if   
+                    end do
+                 end if
+                 
+                 loglikeo=loglikeo+
+     &                    log(2.d0*alpha*dble(je2)+dble(2*countern))-
+     &                    log(2.d0*alpha*dble(je2)+dble(  countero))
+
+                 if(countern.eq.0)then
+                    ok=0
+                  else  
+                    countero=countern
+                    j=j+1
+                 end if   
+              end do
+
+100           continue
+              
+              loglikeo=loglikeo+dnrm(v(i),mu,sd,1)
+         end if
+      end do
+
+
+c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+c++++ Scanning the posterior distribution
+c+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
+
       do iscan=1,nscan
 
 c+++++++ check if the user has requested an interrupt
@@ -328,7 +445,6 @@ c+++++++ evaluate log-prior for current value of parameters
 c+++++++ evaluate log-likelihood
 
          loglikec=0.d0
-         loglikeo=0.d0
 
          do i=1,nrec
          
@@ -340,102 +456,14 @@ c+++++++ evaluate log-likelihood
             end do
             vc(i)=y(i)-tmp1
 
-            tmp1=0.d0
-            do j=1,p
-               tmp1=tmp1+x(i,j)*beta(j)
-            end do
-            v(i)=y(i)-tmp1
-         
-
 c++++++++++ first observation
             if(i.eq.1)then
 
-c+++++++++++++++ candidate value 
                  loglikec=dnrm(vc(1),mu,sd ,1)
 
-c+++++++++++++++ current value 
-                 loglikeo=dnrm(v(1) ,mu,sd ,1)
-
 c++++++++++ following observations
               else
 
-                 quan=0.d0
-                 countero=0
-                 
-                 if(v(i).le.quan) then
-                    do l=1,i-1
-                       if(v(l).le.quan)countero=countero+1
-                    end do
-                  else
-                    do l=1,i-1
-                       if(v(l).gt.quan)countero=countero+1
-                    end do
-                 end if  
-
-                 if(countero.eq.0)go to 100
-                 
-                 ok=1
-                 j=2
-                 do while(ok.eq.1.and.j.le.maxm)
-                    nint=2**j
-                    je2=j**2
-                    prob=1.d0/dble(nint)
-                    k=1
-                    
-                    quan=invcdfnorm(prob,mu,sd,1,0)
-                    
-                    do while(v(i).gt.quan.and.k.le.(nint-1))
-                       k=k+1
-                       if(k.lt.nint)then
-                         quan=invcdfnorm(dble(k)*prob,mu,sd,1,0)
-                       end if  
-                    end do
-                    
-                    countern=0
-                    
-                    if(k.eq.1)then
-                       do l=1,i-1
-                          if(v(l).le.quan)countern=countern+1
-                       end do
-                     else if(k.eq.nint)then
-                       quan=invcdfnorm(dble(k-1)*prob,mu,sd,1,0) 
-                       do l=1,i-1
-                          if(v(l).gt.quan)countern=countern+1
-                       end do
-                     else
-                       tmp1=invcdfnorm(dble(k-1)*prob,mu,sd,1,0)
-                       tmp2=invcdfnorm(dble(k  )*prob,mu,sd,1,0)
-
-                       if(tmp1.ge.tmp2)then
-                         call rexit("Error in the limits")
-                       end if  
-                     
-                       do l=1,i-1
-                          if(v(l).gt.tmp1.and.v(l).le.tmp2)then
-                             countern=countern+1
-                          end if   
-                       end do
-                    end if
-                    
-                    loglikeo=loglikeo+
-     &                       log(2.d0*alpha*dble(je2)+dble(2*countern))-
-     &                       log(2.d0*alpha*dble(je2)+dble(  countero))
-
-                    if(countern.eq.0)then
-                       ok=0
-                     else  
-                       countero=countern
-                       j=j+1
-                    end if   
-                 end do
-
-100              continue
-                 
-                 loglikeo=loglikeo+dnrm(v(i),mu,sd,1)
-
-
-c+++++++++++++++ candidate value 
-
                  quan=0.d0
                  
                  countero=0
@@ -525,7 +553,12 @@ c+++++++ acceptance step
             do j=1,p
                beta(j)=betac(j)
             end do
-
+            
+            do j=1,nrec
+               v(j)=vc(j)
+            end do
+            
+            loglikeo=loglikec
             acrate(1)=acrate(1)+1.d0
          end if
 
@@ -539,11 +572,11 @@ c+++++++ check if the user has requested an interrupt
 	
 c+++++++ sample candidates
 
-         sdc=rtlnorm(log(sd),0.01d0,0,0,.true.,.true.)
+         sdc=rtlnorm(log(sd),tune2*0.1d0,0,0,.true.,.true.)
          sigma2c=sdc**2
 
-         logcgkn=dlnrm(sd,log(sdc),0.01d0,1) 
-         logcgko=dlnrm(sdc,log(sd ),0.01d0,1) 
+         logcgkn=dlnrm(sd,log(sdc),tune2*0.1d0,1) 
+         logcgko=dlnrm(sdc,log(sd ),tune2*0.1d0,1) 
 
 
 c+++++++ evaluate log-prior for candidate value of the parameters
@@ -559,108 +592,19 @@ c+++++++ evaluate log-prior for current value of parameters
 c+++++++ evaluate log-likelihood
 
          loglikec=0.d0
-         loglikeo=0.d0
 
          do i=1,nrec
          
             call rchkusr()   
             
-            tmp1=0.d0
-            do j=1,p
-               tmp1=tmp1+x(i,j)*beta(j)
-            end do
-            v(i)=y(i)-tmp1
-         
-
 c++++++++++ first observation
             if(i.eq.1)then
 
-c+++++++++++++++ candidate value 
                  loglikec=dnrm(v(1) ,mu,sdc,1)
 
-c+++++++++++++++ current value 
-                 loglikeo=dnrm(v(1) ,mu,sd ,1)
-
 c++++++++++ following observations
               else
 
-                 quan=0.d0
-                 countero=0
-                 
-                 if(v(i).le.quan) then
-                    do l=1,i-1
-                       if(v(l).le.quan)countero=countero+1
-                    end do
-                  else
-                    do l=1,i-1
-                       if(v(l).gt.quan)countero=countero+1
-                    end do
-                 end if  
-
-                 if(countero.eq.0)go to 102
-                 
-                 ok=1
-                 j=2
-                 do while(ok.eq.1.and.j.le.maxm)
-                    nint=2**j
-                    je2=j**2
-                    prob=1.d0/dble(nint)
-                    k=1
-                    
-                    quan=invcdfnorm(prob,mu,sd,1,0)
-                    
-                    do while(v(i).gt.quan.and.k.le.(nint-1))
-                       k=k+1
-                       if(k.lt.nint)then
-                         quan=invcdfnorm(dble(k)*prob,mu,sd,1,0)
-                       end if  
-                    end do
-                    
-                    countern=0
-                    
-                    if(k.eq.1)then
-                       do l=1,i-1
-                          if(v(l).le.quan)countern=countern+1
-                       end do
-                     else if(k.eq.nint)then
-                       quan=invcdfnorm(dble(k-1)*prob,mu,sd,1,0) 
-                       do l=1,i-1
-                          if(v(l).gt.quan)countern=countern+1
-                       end do
-                     else
-                       tmp1=invcdfnorm(dble(k-1)*prob,mu,sd,1,0)
-                       tmp2=invcdfnorm(dble(k  )*prob,mu,sd,1,0)
-
-                       if(tmp1.ge.tmp2)then
-                         call rexit("Error in the limits")
-                       end if  
-                     
-                       do l=1,i-1
-                          if(v(l).gt.tmp1.and.v(l).le.tmp2)then
-                             countern=countern+1
-                          end if   
-                       end do
-                    end if
-                    
-                    loglikeo=loglikeo+
-     &                       log(2.d0*alpha*dble(je2)+dble(2*countern))-
-     &                       log(2.d0*alpha*dble(je2)+dble(  countero))
-
-                    if(countern.eq.0)then
-                       ok=0
-                     else  
-                       countero=countern
-                       j=j+1
-                    end if   
-                 end do
-
-102              continue
-                 
-                 loglikeo=loglikeo+dnrm(v(i),mu,sd,1)
-
-
-c+++++++++++++++ candidate value 
-
                  quan=0.d0
                  
                  countero=0
@@ -675,7 +619,7 @@ c+++++++++++++++ candidate value
                      end do
                  end if  
 
-                 if(countero.eq.0)go to 103
+                 if(countero.eq.0)go to 102
 
                  ok=1
                  j=2
@@ -733,7 +677,7 @@ c+++++++++++++++ candidate value
                     end if   
                  end do
 
-103              continue
+102              continue
                  
                  loglikec=loglikec+dnrm(v(i),mu,sdc,1)
 
@@ -750,10 +694,10 @@ c+++++++ acceptance step
          if(dble(runif()).lt.ratio)then
             sd=sdc
             sigma2=sd**2
+            loglikeo=loglikec
             acrate(2)=acrate(2)+1.d0
          end if
 
-
 c++++++++++++++++++++++++++++++++++         
 c+++++++ Precision parameter
 c++++++++++++++++++++++++++++++++++
@@ -765,9 +709,9 @@ c+++++++ check if the user has requested an interrupt
 	
 c+++++++++++ sample candidates
 
-             alphac=rtlnorm(log(alpha),0.1d0,0,0,.true.,.true.)
-             logcgkn=dlnrm(alpha ,log(alphac),0.1d0,1) 
-             logcgko=dlnrm(alphac,log(alpha ),0.1d0,1) 
+             alphac=rtlnorm(log(alpha),tune3*0.1d0,0,0,.true.,.true.)
+             logcgkn=dlnrm(alpha ,log(alphac),tune3*0.1d0,1) 
+             logcgko=dlnrm(alphac,log(alpha ),tune3*0.1d0,1) 
 
 c+++++++++++ evaluate log-prior for candidate value of the parameters
 
@@ -780,107 +724,19 @@ c+++++++++++ evaluate log-prior for current value of parameters
 c+++++++++++ evaluate log-likelihood
 
              loglikec=0.d0
-             loglikeo=0.d0
 
              do i=1,nrec
          
                 call rchkusr()   
             
-                tmp1=0.d0
-                do j=1,p
-                   tmp1=tmp1+x(i,j)*beta(j)
-                end do
-                v(i)=y(i)-tmp1
-         
-
 c++++++++++++++ first observation
                 if(i.eq.1)then
 
-c+++++++++++++++++++ candidate value 
                      loglikec=dnrm(v(1) ,mu,sd,1)
 
-c+++++++++++++++++++ current value 
-                     loglikeo=dnrm(v(1) ,mu,sd,1)
-
 c++++++++++++++ following observations
                   else
 
-                     quan=0.d0
-                     countero=0
-                 
-                     if(v(i).le.quan) then
-                        do l=1,i-1
-                           if(v(l).le.quan)countero=countero+1
-                        end do
-                      else
-                        do l=1,i-1
-                           if(v(l).gt.quan)countero=countero+1
-                        end do
-                     end if  
-
-                     if(countero.eq.0)go to 104
-                 
-                     ok=1
-                     j=2
-                     do while(ok.eq.1.and.j.le.maxm)
-                        nint=2**j
-                        je2=j**2
-                        prob=1.d0/dble(nint)
-                        k=1
-                    
-                        quan=invcdfnorm(prob,mu,sd,1,0)
-                    
-                        do while(v(i).gt.quan.and.k.le.(nint-1))
-                           k=k+1
-                           if(k.lt.nint)then
-                             quan=invcdfnorm(dble(k)*prob,mu,sd,1,0)
-                           end if  
-                        end do
-                    
-                        countern=0
-                    
-                        if(k.eq.1)then
-                           do l=1,i-1
-                              if(v(l).le.quan)countern=countern+1
-                           end do
-                         else if(k.eq.nint)then
-                           quan=invcdfnorm(dble(k-1)*prob,mu,sd,1,0) 
-                           do l=1,i-1
-                              if(v(l).gt.quan)countern=countern+1
-                           end do
-                         else
-                           tmp1=invcdfnorm(dble(k-1)*prob,mu,sd,1,0)
-                           tmp2=invcdfnorm(dble(k  )*prob,mu,sd,1,0)
-
-                           if(tmp1.ge.tmp2)then
-                             call rexit("Error in the limits")
-                           end if  
-                     
-                           do l=1,i-1
-                              if(v(l).gt.tmp1.and.v(l).le.tmp2)then
-                                 countern=countern+1
-                              end if   
-                           end do
-                        end if
-                    
-                        loglikeo=loglikeo+
-     &                       log(2.d0*alpha*dble(je2)+dble(2*countern))-
-     &                       log(2.d0*alpha*dble(je2)+dble(  countero))
-
-                        if(countern.eq.0)then
-                           ok=0
-                          else  
-                           countero=countern
-                           j=j+1
-                        end if   
-                     end do
-
-104                  continue
-                 
-                     loglikeo=loglikeo+dnrm(v(i),mu,sd,1)
-
-c+++++++++++++++++++ candidate value 
-
                      quan=0.d0
                  
                      countero=0
@@ -895,7 +751,7 @@ c+++++++++++++++++++ candidate value
                          end do
                      end if  
 
-                     if(countero.eq.0)go to 105
+                     if(countero.eq.0)go to 103
 
                      ok=1
                      j=2
@@ -953,7 +809,7 @@ c+++++++++++++++++++ candidate value
                         end if   
                      end do
 
-105                  continue
+103                  continue
                  
                      loglikec=loglikec+dnrm(v(i),mu,sdc,1)
 
@@ -970,6 +826,7 @@ c+++++++++++ acceptance step
              if(dble(runif()).lt.ratio)then
                 alpha=alphac
                 acrate(3)=acrate(3)+1.d0
+                loglikeo=loglikec
              end if
          end if 
 
@@ -1119,35 +976,28 @@ c+++++++++++++ Now j indicates the partition and k the interval
 c+++++++++++++ cpo
 
                do i=1,nrec
-                  loglikeo=0.d0
-
-                  tmp1=0.d0
-                  
-                  do j=1,p
-                     tmp1=tmp1+x(i,j)*beta(j)
-                  end do
-                  v(i)=y(i)-tmp1
+                  loglikec=0.d0
 
                   quan=0.d0
                   denom=1.0d0
                   
                   countero=0
-                 
+
                   if(v(i).le.quan) then
                       do l=1,nrec
-                         if(v(l).le.quan)countero=countero+1
+                         if(v(l).le.quan.and.l.ne.i)countero=countero+1
                       end do
                     else
                       do l=1,nrec
-                         if(v(l).gt.quan)countero=countero+1
+                         if(v(l).gt.quan.and.l.ne.i)countero=countero+1
                       end do
                   end if  
-
-                  if(countero.eq.0)go to 106
+                 
+                  if(countero.eq.0)go to 104
 
                   ok=1
                   j=2
-                  do while(ok.eq.1.and.j.le.6)
+                  do while(ok.eq.1.and.j.le.maxm)
                      nint=2**j
                      je2=j**2
                      prob=1.d0/dble(nint)
@@ -1164,6 +1014,104 @@ c+++++++++++++ cpo
                         end if  
                      end do
                     
+                     countern=0
+
+                     if(k.eq.1)then
+                        do l=1,nrec
+                           if(v(l).le.quan.and.l.ne.i)then
+                              countern=countern+1
+                           end if   
+                        end do
+                      else if(k.eq.nint)then
+                        quan=invcdfnorm(dble(k-1)/dble(nint),mu,sd,1,0) 
+                        do l=1,nrec
+                           if(v(l).gt.quan.and.l.ne.i)then
+                              countern=countern+1
+                           end if   
+                        end do
+                      else
+                        tmp1=invcdfnorm(dble(k-1)/dble(nint),mu,sd,1,0)
+                        tmp2=invcdfnorm(dble(k  )/dble(nint),mu,sd,1,0)
+
+                        if(tmp1.ge.tmp2)then
+                          call rexit("Error in the limits")
+                        end if  
+                     
+                        do l=1,nrec
+                           if(l.ne.i)then
+                           if(v(l).gt.tmp1.and.v(l).le.tmp2)then
+                             countern=countern+1
+                           end if
+                           end if
+                        end do
+                     end if
+                    
+                     loglikec=loglikec+
+     &                   log(2.d0)+                
+     &                   log(     alpha*dble(je2)+dble(  countern))-
+     &                   log(2.d0*alpha*dble(je2)+dble(  countero))
+
+
+                     if(countern.eq.0)then
+                         ok=0
+                       else  
+                         countero=countern
+                         j=j+1
+                     end if   
+                  end do
+
+104               continue
+                 
+                  
+                  loglikec=loglikec+dnrm(v(i),mu,sd,1)
+
+                  cpo(i)=cpo(i)+exp(loglikec)  
+            
+               end do
+
+
+c+++++++++++++ density estimate
+
+               do i=1,ngrid
+               
+                  loglikec=0.d0
+
+                  quan=0.d0
+                  denom=1.0d0
+                  
+                  countero=0
+                 
+                  if(grid(i).le.quan) then
+                      do l=1,nrec
+                         if(v(l).le.quan)countero=countero+1
+                      end do
+                    else
+                      do l=1,nrec
+                         if(v(l).gt.quan)countero=countero+1
+                      end do
+                  end if  
+
+                  if(countero.eq.0)go to 105
+
+                  ok=1
+                  j=2
+                  do while(ok.eq.1.and.j.le.maxm)
+                     nint=2**j
+                     je2=j**2
+                     prob=1.d0/dble(nint)
+                     k=1
+
+                     denom=prob
+                     
+                     quan=invcdfnorm(prob,mu,sd,1,0)
+                    
+                     do while(grid(i).gt.quan.and.k.le.(nint-1))
+                        k=k+1
+                        if(k.lt.nint)then
+                          quan=invcdfnorm(dble(k)/dble(nint),mu,sd,1,0)
+                        end if  
+                     end do
+                    
                      countern=0
                     
                      if(k.eq.1)then
@@ -1190,7 +1138,7 @@ c+++++++++++++ cpo
                         end do
                      end if
                     
-                     loglikeo=loglikeo+
+                     loglikec=loglikec+
      &                   log(2.d0)+                
      &                   log(     alpha*dble(je2)+dble(  countern))-
      &                   log(2.d0*alpha*dble(je2)+dble(  countero))
@@ -1204,13 +1152,12 @@ c+++++++++++++ cpo
                      end if   
                   end do
 
-106               continue
+105               continue
                  
                   
-                  loglikeo=loglikeo+dnrm(v(i),mu,sd,1)
+                  loglikec=loglikec+dnrm(grid(i),mu,sd,1)
 
-                  cpo(i)=cpo(i)+1.0d0/exp(loglikeo)  
-            
+                  f(i)=f(i)+exp(loglikec)  
                end do
 
 c+++++++++++++ print
@@ -1236,8 +1183,13 @@ c++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
       acrate(3)=acrate(3)/dble(nscan)        
       
       do i=1,nrec
-         cpo(i)=dble(nsave)/cpo(i)
+         cpo(i)=cpo(i)/dble(nsave)
       end do
+
+      do i=1,ngrid
+         f(i)=f(i)/dble(nsave)
+      end do
+
       
       return
       end
